@@ -2,7 +2,10 @@ from pathlib import Path
 from typing import cast
 
 from recon.corpus import Corpus
-from recon.stats import calculate_label_distribution_similarity, get_ner_stats
+from recon.stats import (
+    calculate_label_distribution_similarity,
+    get_ner_stats,
+)
 from recon.types import NERStats
 from wasabi import Printer
 
