@@ -1,8 +1,13 @@
-import hashlib
-from typing import Any, Callable, Tuple, Union
+from typing import TYPE_CHECKING, Callable, Tuple, Union
+
+import xxhash
+
+if TYPE_CHECKING:
+    from recon.dataset import Dataset
+    from recon.types import Example, PredictionError, Span, Token
 
 
-def token_hash(token: Any, as_int: bool = True) -> Union[str, int]:
+def token_hash(token: "Token", as_int: bool = True) -> Union[str, int]:
     """Hash of Token type
 
     Args:
@@ -15,7 +20,7 @@ def token_hash(token: Any, as_int: bool = True) -> Union[str, int]:
     return _hash((token.text, token.start, token.end, token.id), as_int=as_int)
 
 
-def span_hash(span: Any, as_int: bool = True) -> Union[str, int]:
+def span_hash(span: "Span", as_int: bool = True) -> Union[str, int]:
     """Hash of Span type
 
     Args:
@@ -36,7 +41,7 @@ def span_hash(span: Any, as_int: bool = True) -> Union[str, int]:
     return _hash(hash_data, as_int=as_int)
 
 
-def example_hash(example: Any, as_int: bool = True) -> Union[str, int]:
+def example_hash(example: "Example", as_int: bool = True) -> Union[str, int]:
     """Hash of Example type
 
     Args:
@@ -50,7 +55,7 @@ def example_hash(example: Any, as_int: bool = True) -> Union[str, int]:
     return _hash(hash_data, as_int=as_int)
 
 
-def tokenized_example_hash(example: Any, as_int: bool = True) -> Union[str, int]:
+def tokenized_example_hash(example: "Example", as_int: bool = True) -> Union[str, int]:
     """Hash of Example type including token data
 
     Args:
@@ -69,7 +74,7 @@ def tokenized_example_hash(example: Any, as_int: bool = True) -> Union[str, int]
     return _hash(hash_data, as_int=as_int)
 
 
-def dataset_hash(dataset: Any, as_int: bool = True) -> Union[str, int]:
+def dataset_hash(dataset: "Dataset", as_int: bool = True) -> Union[str, int]:
     """Hash of Dataset
 
     Args:
@@ -79,15 +84,25 @@ def dataset_hash(dataset: Any, as_int: bool = True) -> Union[str, int]:
     Returns:
         Union[str, int]: Dataset hash
     """
-    hash_data = (dataset.name,) + tuple(
-        (example_hash(example, as_int=False) for example in dataset.data)
-    )
+    hash_data = (dataset.name,) + tuple((example_hash(example, as_int=False) for example in dataset.data))
     return _hash(hash_data, as_int=as_int)
 
 
-def _hash(
-    tpl: Tuple, hash_function: Callable = hashlib.sha1, as_int: bool = True
-) -> Union[str, int]:
+def prediction_error_hash(prediction_error: "PredictionError", as_int: bool = True) -> Union[str, int]:
+    """Hash of PredictionError
+
+    Args:
+        prediction_error (PredictionError): PredictionError to hash
+        as_int (bool, optional): Encode hash as int
+
+    Returns:
+        Union[str, int]: PredictionError hash
+    """
+    hash_data = (prediction_error.text, prediction_error.true_label, prediction_error.pred_label)
+    return _hash(hash_data, as_int=as_int)
+
+
+def _hash(tpl: Tuple, hash_function: Callable = xxhash.xxh64, as_int: bool = True) -> Union[str, int]:
     """Deterministic hash function. The main use here is
     providing a `commit_hash` for a Dataset to compare across
     saves/loads and ensure that operations are re-run if the hash
@@ -109,10 +124,4 @@ def _hash(
         else:
             e_bytes = bytes(e)
         m.update(e_bytes)
-
-    hd = m.hexdigest()
-
-    if as_int:
-        return int(hd, 16)
-    else:
-        return hd
+    return m.intdigest() if as_int else m.hexdigest()
