@@ -1,21 +1,21 @@
 from collections import defaultdict
-from typing import DefaultDict, Dict, List, Set
+from typing import DefaultDict, Dict, List, Set, Tuple
 
-from recon.constants import NONE
+import numpy as np
+from recon.constants import NOT_LABELED
 from recon.recognizer import EntityRecognizer
 from recon.types import (
     Example,
-    HardestExample,
+    HardestExampleV2,
     LabelDisparity,
     PredictionError,
     PredictionErrorExamplePair,
 )
+from spacy.scorer import PRFScore
 from wasabi import Printer
 
 
-def get_ents_by_label(
-    data: List[Example], case_sensitive: bool = False
-) -> DefaultDict[str, List[str]]:
+def get_ents_by_label(data: List[Example], case_sensitive: bool = False) -> DefaultDict[str, List[str]]:
     """Get a dictionary of unique text spans by label for your data
 
     # TODO: Ok so this needs to return more than just a set for each label.
@@ -52,9 +52,7 @@ def get_ents_by_label(
     return sorted_annotations
 
 
-def get_label_disparities(
-    data: List[Example], label1: str, label2: str, case_sensitive: bool = False
-) -> Set[str]:
+def get_label_disparities(data: List[Example], label1: str, label2: str, case_sensitive: bool = False) -> Set[str]:
     """Identify annotated spans that have different labels in different examples
 
     Args:
@@ -99,9 +97,7 @@ def top_label_disparities(
                     else:
                         input_hash = "||".join([label1, label2])
 
-                    label_disparities[input_hash] = LabelDisparity(
-                        label1=label1, label2=label2, count=n_disparities
-                    )
+                    label_disparities[input_hash] = LabelDisparity(label1=label1, label2=label2, count=n_disparities)
 
     return sorted(label_disparities.values(), key=lambda ld: ld.count, reverse=True)
 
@@ -142,16 +138,14 @@ def top_prediction_errors(
     anns = (e.spans for e in data)
 
     errors = defaultdict(lambda: defaultdict(lambda: defaultdict(int)))  # type: ignore
-    error_examples: DefaultDict[str, List[PredictionErrorExamplePair]] = defaultdict(list)
+    error_examples: DefaultDict[Tuple[str, str, str], List[PredictionErrorExamplePair]] = defaultdict(list)
     n_errors = 0
 
     for orig_example, pred_example, ann in zip(data, recognizer.predict(texts), anns):
         if k is not None and n_errors > k:
             break
 
-        pred_error_example_pair = PredictionErrorExamplePair(
-            original=orig_example, predicted=pred_example
-        )
+        pred_error_example_pair = PredictionErrorExamplePair(original=orig_example, predicted=pred_example)
 
         cand = set([(s.start, s.end, s.label) for s in pred_example.spans])
         gold = set([(s.start, s.end, s.label) for s in ann])
@@ -173,14 +167,12 @@ def top_prediction_errors(
                     text = pred_example.text[start:end]
                     false_label = fp[2]
                     errors[label][text][false_label] += 1
-                    error_examples[f"{text}||{label}||{false_label}"].append(
-                        pred_error_example_pair
-                    )
+                    error_examples[(text, label, false_label)].append(pred_error_example_pair)
                 else:
                     start, end, false_label = fp
                     text = pred_example.text[start:end]
-                    errors[NONE][text][false_label] += 1
-                    error_examples[f"{text}||{NONE}||{false_label}"].append(pred_error_example_pair)
+                    errors[NOT_LABELED][text][false_label] += 1
+                    error_examples[(text, NOT_LABELED, false_label)].append(pred_error_example_pair)
                 n_errors += 1
                 seen.add((start, end))
 
@@ -189,22 +181,22 @@ def top_prediction_errors(
                 start, end, label = fn
                 if (start, end) not in seen:
                     text = pred_example.text[start:end]
-                    errors[label][text][NONE] += 1
-                    error_examples[f"{text}||{label}||{NONE}"].append(pred_error_example_pair)
+                    errors[label][text][NOT_LABELED] += 1
+                    error_examples[(text, label, NOT_LABELED)].append(pred_error_example_pair)
                     n_errors += 1
 
-    ranked_errors_map: Dict[str, PredictionError] = {}
+    ranked_errors_map: Dict[Tuple[str, str, str], PredictionError] = {}
 
     for label, errors_per_label in errors.items():
         for error_text, error_labels in errors_per_label.items():
             for error_label, count in error_labels.items():
-                pe_hash = f"{error_text}||{label}||{error_label}"
+                pe_hash = (error_text, label, error_label)
                 ranked_errors_map[pe_hash] = PredictionError(
                     text=error_text,
                     true_label=label,
                     pred_label=error_label,
                     count=count,
-                    examples=error_examples[f"{error_text}||{label}||{error_label}"],
+                    examples=error_examples[pe_hash],
                 )
 
     ranked_errors: List[PredictionError] = sorted(
@@ -232,85 +224,57 @@ def top_prediction_errors(
 
 
 def get_hardest_examples(
-    pred_errors: List[PredictionError],
-    return_pred_errors: bool = True,
-    remove_pred_error_examples: bool = True,
-) -> List[HardestExample]:
-    """Get hardest examples from list of PredictionError types
+    recognizer: EntityRecognizer,
+    examples: List[Example],
+    score_count: bool = True,
+    normalize_scores: bool = True,
+) -> List[HardestExampleV2]:
+    """Get hardest examples for a recognizer to predict on and sort by difficulty with the goal
+    of quickly identifying the biggest holes in a model / annotated data.
 
     Args:
-        pred_errors (List[PredictionError]): list of PredictionError
-        return_pred_errors (bool, optional): Whether to return prediction errors. Defaults to True.
-        remove_pred_error_examples (bool, optional): Whether to remove examples from returned PredictionError. Defaults to True.
-
-    Raises:
-        ValueError: Each PredictionError must have a List of examples
+        recognizer (EntityRecognizer): EntityRecognizer to test predictions for
+        examples (List[Example]): Set of input examples
+        score_count (bool): Adjust score by total number of errors
+        normalize_scores (bool): Scale scores adjusted by count between 0 and 1
 
     Returns:
-        List[HardestExample]: Sorted list of the hardest examples for a model to work on.
+        List[HardestExampleV2]: HardestExamples sorted by difficulty (hardest first)
     """
+    preds = recognizer.predict((e.text for e in examples))
 
-    has_examples = any([pe.examples for pe in pred_errors])
-    if not has_examples:
-        raise ValueError(
-            "Each PredictionError in Parameter pred_errors must have examples attached."
+    max_count = 0
+    hes = []
+    for pred, ref in zip(preds, examples):
+
+        scorer = PRFScore()
+        scorer.score_set(
+            set([(s.start, s.end, s.label) for s in pred.spans]),
+            set([(s.start, s.end, s.label) for s in ref.spans]),
         )
+        total_errors = scorer.fp + scorer.fn
+        score = scorer.fscore if (pred.spans and ref.spans) else 1.0
 
-    examples_text_map: Dict[str, Example] = {}
-    example_pred_errors_map: DefaultDict[str, List[PredictionError]] = defaultdict(list)
-    for pe in pred_errors:
-        if pe.examples:
-            for example in pe.examples:
-                examples_text_map[example.original.text] = example.original
-                example_pred_errors_map[example.original.text].append(
-                    PredictionError(
-                        text=pe.text,
-                        true_label=pe.true_label,
-                        pred_label=pe.pred_label,
-                        count=pe.count,
-                        examples=[example],
-                    )
-                )
+        if total_errors > max_count:
+            max_count = total_errors
 
-    hardest_examples = []
-    for example_text, example_pred_errors in example_pred_errors_map.items():
-        example = examples_text_map[example_text]  # type: ignore
+        he = HardestExampleV2(reference=ref, prediction=pred, count=total_errors, score=score)
+        hes.append(he)
 
-        prediction_errors: List[PredictionError] = []
-        if remove_pred_error_examples and example_pred_errors:
-            prediction_errors = [
-                PredictionError(
-                    text=pe.text,
-                    true_label=pe.true_label,
-                    pred_label=pe.pred_label,
-                    count=pe.count,
-                    examples=[],
-                )
-                for pe in example_pred_errors
-            ]
-        else:
-            prediction_errors = example_pred_errors
+    if score_count:
+        for he in hes:
+            he.score -= he.count / max_count
+        if normalize_scores:
+            scores = np.asarray([he.score for he in hes])
+            scores = (scores - scores.min()) / np.ptp(scores)
+            for i, he in enumerate(hes):
+                he.score = scores[i]
 
-        prediction_error_hashes: Set[str] = set()
-        deduped_prediction_errors: List[PredictionError] = []
-
-        for pe in prediction_errors:
-            pe_hash = f"{pe.text}||{pe.true_label}||{pe.pred_label}"
-            if pe_hash not in prediction_error_hashes:
-                deduped_prediction_errors.append(pe)
-
-        record = HardestExample(example=example, count=len(deduped_prediction_errors))
-        if return_pred_errors:
-            record.prediction_errors = deduped_prediction_errors
-        hardest_examples.append(record)
-
-    sorted_hardest_examples = sorted(hardest_examples, key=lambda he: he.count, reverse=True)
-    return sorted_hardest_examples
+    sorted_hes = sorted(hes, key=lambda he: (he.score, he.count))
+    return sorted_hes
 
 
-def get_annotation_labels(
-    examples: List[Example], case_sensitive: bool = False
-) -> Dict[str, Dict[str, list]]:
+def get_annotation_labels(examples: List[Example], case_sensitive: bool = False) -> Dict[str, Dict[str, list]]:
     """Constructs a map of each annotation in the list of examples to each label that annotation
     has and references all examples associated with that label.
 
