@@ -1,10 +1,10 @@
 import math
 from collections import defaultdict
-from typing import Any, DefaultDict, Dict, List, Sequence, Union, cast
+from typing import Any, DefaultDict, Dict, List, Optional, Sequence, Union, cast
 
 import numpy as np
 import srsly
-from recon.constants import NONE
+from recon.constants import NOT_LABELED
 from recon.types import EntityCoverage, EntityCoverageStats, Example, NERStats, Outliers
 from scipy.spatial.distance import jensenshannon
 from scipy.stats import entropy as scipy_entropy
@@ -34,15 +34,13 @@ def get_ner_stats(
         length += 1
         if not e.spans:
             n_examples_no_entities += 1
-            examples[NONE].append(e)
+            examples[NOT_LABELED].append(e)
         else:
             for s in e.spans:
                 annotations_per_type[s.label] += 1
                 examples[s.label].append(e)
 
-    sorted_anns_by_count = {
-        a[0]: a[1] for a in sorted(annotations_per_type.items(), key=lambda x: x[1], reverse=True)
-    }
+    sorted_anns_by_count = {a[0]: a[1] for a in sorted(annotations_per_type.items(), key=lambda x: x[1], reverse=True)}
 
     stats = NERStats(
         n_examples=length,
@@ -54,7 +52,7 @@ def get_ner_stats(
         stats.examples_with_type = examples
 
     if serialize:
-        return srsly.json_dumps(stats.dict(), indent=4)
+        return stats.json(indent=4)
     else:
         return stats
 
@@ -70,8 +68,7 @@ def get_sorted_type_counts(ner_stats: NERStats) -> List[int]:
         List[int]: List of counts sorted by type name
     """
     annotations_per_type = ner_stats.n_annotations_per_type
-    annotations_per_type[NONE] = ner_stats.n_examples_no_entities
-
+    annotations_per_type[NOT_LABELED] = ner_stats.n_examples_no_entities
     return [t[1] for t in sorted(annotations_per_type.items(), key=lambda p: p[0])]
 
 
